@@ -3,15 +3,18 @@ package controlapi
 import (
 	"context"
 	"fmt"
+	"reflect"
 	"strings"
 	"testing"
 	"time"
 
 	"github.com/docker/swarmkit/api"
+	"github.com/docker/swarmkit/identity"
 	"github.com/docker/swarmkit/manager/state/store"
 	"github.com/docker/swarmkit/testutils"
 	gogotypes "github.com/gogo/protobuf/types"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 	"google.golang.org/grpc/codes"
 )
 
@@ -214,12 +217,22 @@ func TestValidateMode(t *testing.T) {
 	bad := []*api.ServiceSpec{
 		// -4 jammed into the replicas field, underflowing the uint64
 		{Mode: &api.ServiceSpec_Replicated{Replicated: &api.ReplicatedService{Replicas: uint64(negative)}}},
+		{Mode: &api.ServiceSpec_ReplicatedJob{ReplicatedJob: &api.ReplicatedJob{MaxConcurrent: uint64(negative)}}},
+		{Mode: &api.ServiceSpec_ReplicatedJob{ReplicatedJob: &api.ReplicatedJob{TotalCompletions: uint64(negative)}}},
 		{},
 	}
 
 	good := []*api.ServiceSpec{
 		{Mode: &api.ServiceSpec_Replicated{Replicated: &api.ReplicatedService{Replicas: 2}}},
 		{Mode: &api.ServiceSpec_Global{}},
+		{
+			Mode: &api.ServiceSpec_ReplicatedJob{
+				ReplicatedJob: &api.ReplicatedJob{
+					MaxConcurrent: 3, TotalCompletions: 9,
+				},
+			},
+		},
+		{Mode: &api.ServiceSpec_GlobalJob{}},
 	}
 
 	for _, b := range bad {
@@ -229,7 +242,8 @@ func TestValidateMode(t *testing.T) {
 	}
 
 	for _, g := range good {
-		assert.NoError(t, validateMode(g))
+		err := validateMode(g)
+		assert.NoError(t, err)
 	}
 }
 
@@ -442,6 +456,71 @@ func TestValidateServiceSpec(t *testing.T) {
 	}
 }
 
+// TestValidateServiceSpecJobsDifference is different from
+// TestValidateServiceSpec in that it checks that job-mode services are
+// validated differently from regular services.
+func TestValidateServiceSpecJobsDifference(t *testing.T) {
+	// correctly formed spec should be valid
+	cannedSpec := createSpec("name", "image", 1)
+	err := validateServiceSpec(cannedSpec)
+	assert.NoError(t, err)
+
+	// Replicated job should not be allowed to have update config
+	specReplicatedJobUpdate := cannedSpec.Copy()
+	specReplicatedJobUpdate.Mode = &api.ServiceSpec_ReplicatedJob{
+		ReplicatedJob: &api.ReplicatedJob{},
+	}
+	specReplicatedJobUpdate.Update = &api.UpdateConfig{}
+	err = validateServiceSpec(specReplicatedJobUpdate)
+	assert.Error(t, err)
+
+	specReplicatedJobNoUpdate := specReplicatedJobUpdate.Copy()
+	specReplicatedJobNoUpdate.Update = nil
+	err = validateServiceSpec(specReplicatedJobNoUpdate)
+	assert.NoError(t, err)
+
+	// Global job should not be allowed to have update config
+	specGlobalJobUpdate := cannedSpec.Copy()
+	specGlobalJobUpdate.Mode = &api.ServiceSpec_GlobalJob{
+		GlobalJob: &api.GlobalJob{},
+	}
+	specGlobalJobUpdate.Update = &api.UpdateConfig{}
+	err = validateServiceSpec(specGlobalJobUpdate)
+	assert.Error(t, err)
+
+	specGlobalJobNoUpdate := specGlobalJobUpdate.Copy()
+	specGlobalJobNoUpdate.Update = nil
+	err = validateServiceSpec(specReplicatedJobNoUpdate)
+	assert.NoError(t, err)
+
+	// Replicated service should be allowed to have update config, which should
+	// be verified for correctness
+	replicatedServiceBrokenUpdate := cannedSpec.Copy()
+	replicatedServiceBrokenUpdate.Update = &api.UpdateConfig{
+		Delay: -1 * time.Second,
+	}
+	err = validateServiceSpec(replicatedServiceBrokenUpdate)
+	assert.Error(t, err)
+
+	replicatedServiceCorrectUpdate := replicatedServiceBrokenUpdate.Copy()
+	replicatedServiceCorrectUpdate.Update.Delay = time.Second
+	err = validateServiceSpec(replicatedServiceCorrectUpdate)
+	assert.NoError(t, err)
+
+	// Global service should be allowed to have update config, which should be
+	// verified for correctness
+	globalServiceBrokenUpdate := replicatedServiceBrokenUpdate.Copy()
+	globalServiceBrokenUpdate.Mode = &api.ServiceSpec_Global{
+		Global: &api.GlobalService{},
+	}
+	err = validateServiceSpec(globalServiceBrokenUpdate)
+	assert.Error(t, err)
+
+	globalServiceCorrectUpdate := globalServiceBrokenUpdate.Copy()
+	globalServiceCorrectUpdate.Update.Delay = time.Second
+	err = validateServiceSpec(globalServiceCorrectUpdate)
+}
+
 func TestValidateRestartPolicy(t *testing.T) {
 	bad := []*api.RestartPolicy{
 		{
@@ -1383,3 +1462,504 @@ func TestListServices(t *testing.T) {
 	assert.NoError(t, err)
 	assert.Equal(t, 0, len(r.Services))
 }
+
+func TestListServiceStatuses(t *testing.T) {
+	ts := newTestServer(t)
+	defer ts.Stop()
+
+	// Test listing no services is empty and has no error
+	r, err := ts.Client.ListServiceStatuses(
+		context.Background(),
+		&api.ListServiceStatusesRequest{},
+	)
+	assert.NoError(t, err, "error when listing no services against an empty store")
+	assert.NotNil(t, r, "response against an empty store was nil")
+	assert.Empty(t, r.Statuses, "response statuses was not empty")
+
+	// Test listing services that do not exist. We should still get a response,
+	// but both desired and actual should be 0
+	r, err = ts.Client.ListServiceStatuses(
+		context.Background(),
+		&api.ListServiceStatusesRequest{Services: []string{"foo"}},
+	)
+	assert.NoError(t, err, "error listing services that do not exist")
+	assert.NotNil(t, r, "response for nonexistant services was nil")
+	assert.Len(t, r.Statuses, 1, "expected 1 status")
+	assert.Equal(
+		t, r.Statuses[0],
+		&api.ListServiceStatusesResponse_ServiceStatus{ServiceID: "foo"},
+	)
+
+	// now test that listing service statuses actually works.
+
+	// justRight will be converged
+	justRight := createService(t, ts, "justRight", "image", 3)
+	// notEnough will not have enough tasks in running
+	notEnough := createService(t, ts, "notEnough", "image", 7)
+
+	// no shortcut for creating a global service
+	globalSpec := createSpec("global", "image", 0)
+	globalSpec.Mode = &api.ServiceSpec_Global{Global: &api.GlobalService{}}
+
+	svcResp, svcErr := ts.Client.CreateService(
+		context.Background(), &api.CreateServiceRequest{Spec: globalSpec},
+	)
+	assert.NoError(t, svcErr)
+
+	// global will have the right number of tasks
+	global := svcResp.Service
+
+	global2Spec := globalSpec.Copy()
+	global2Spec.Annotations.Name = "global2"
+
+	svcResp, svcErr = ts.Client.CreateService(
+		context.Background(), &api.CreateServiceRequest{Spec: global2Spec},
+	)
+	assert.NoError(t, svcErr)
+
+	// global2 will not have enough tasks
+	global2 := svcResp.Service
+
+	// over will have too many tasks running (as would be seen in a scale-down
+	over := createService(t, ts, "over", "image", 2)
+
+	// replicatedJob1 will be partly completed
+	replicatedJob1Spec := createSpec("replicatedJob1", "image", 0)
+	replicatedJob1Spec.Mode = &api.ServiceSpec_ReplicatedJob{
+		ReplicatedJob: &api.ReplicatedJob{
+			MaxConcurrent:    2,
+			TotalCompletions: 10,
+		},
+	}
+
+	svcResp, svcErr = ts.Client.CreateService(
+		context.Background(), &api.CreateServiceRequest{Spec: replicatedJob1Spec},
+	)
+	assert.NoError(t, svcErr)
+	assert.NotNil(t, svcResp)
+	replicatedJob1 := svcResp.Service
+
+	// replicatedJob2 has been executed before, and will have tasks from a
+	// previous JobIteration
+	replicatedJob2Spec := replicatedJob1Spec.Copy()
+	replicatedJob2Spec.Annotations.Name = "replicatedJob2"
+	svcResp, svcErr = ts.Client.CreateService(
+		context.Background(), &api.CreateServiceRequest{Spec: replicatedJob2Spec},
+	)
+	assert.NoError(t, svcErr)
+	assert.NotNil(t, svcResp)
+	replicatedJob2 := svcResp.Service
+
+	// globalJob is a partly complete global job
+	globalJobSpec := createSpec("globalJob", "image", 0)
+	globalJobSpec.Mode = &api.ServiceSpec_GlobalJob{
+		GlobalJob: &api.GlobalJob{},
+	}
+	svcResp, svcErr = ts.Client.CreateService(
+		context.Background(), &api.CreateServiceRequest{Spec: globalJobSpec},
+	)
+	assert.NoError(t, svcErr)
+	assert.NotNil(t, svcResp)
+	globalJob := svcResp.Service
+
+	// now create some tasks. use a quick helper function for this
+	createTask := func(s *api.Service, actual api.TaskState, desired api.TaskState, opts ...func(*api.Service, *api.Task)) *api.Task {
+		task := &api.Task{
+			ID:           identity.NewID(),
+			DesiredState: desired,
+			Spec:         s.Spec.Task,
+			Status: api.TaskStatus{
+				State: actual,
+			},
+			ServiceID: s.ID,
+		}
+
+		for _, opt := range opts {
+			opt(s, task)
+		}
+
+		err := ts.Store.Update(func(tx store.Tx) error {
+			return store.CreateTask(tx, task)
+		})
+		assert.NoError(t, err)
+		return task
+	}
+
+	withJobIteration := func(s *api.Service, task *api.Task) {
+		assert.NotNil(t, s.JobStatus)
+		task.JobIteration = &(s.JobStatus.JobIteration)
+	}
+
+	// alias task states for brevity
+	running := api.TaskStateRunning
+	shutdown := api.TaskStateShutdown
+	completed := api.TaskStateCompleted
+	newt := api.TaskStateNew
+	failed := api.TaskStateFailed
+
+	// create 3 running tasks for justRight
+	for i := 0; i < 3; i++ {
+		createTask(justRight, running, running)
+	}
+	// create 2 failed and 2 shutdown tasks
+	for i := 0; i < 2; i++ {
+		createTask(justRight, failed, shutdown)
+		createTask(justRight, shutdown, shutdown)
+	}
+
+	// create 4 tasks for notEnough
+	for i := 0; i < 4; i++ {
+		createTask(notEnough, running, running)
+	}
+	// create 3 tasks in new state
+	for i := 0; i < 3; i++ {
+		createTask(notEnough, newt, running)
+	}
+	// create 1 failed and 1 shutdown task
+	createTask(notEnough, failed, shutdown)
+	createTask(notEnough, shutdown, shutdown)
+
+	// create 2 tasks out of 2 desired for global
+	for i := 0; i < 2; i++ {
+		createTask(global, running, running)
+	}
+	// create 3 shutdown tasks for global
+	for i := 0; i < 3; i++ {
+		createTask(global, shutdown, shutdown)
+	}
+
+	// create 4 out of 5 tasks for global2
+	for i := 0; i < 4; i++ {
+		createTask(global2, running, running)
+	}
+	createTask(global2, newt, running)
+
+	// create 6 failed tasks
+	for i := 0; i < 6; i++ {
+		createTask(global2, failed, shutdown)
+	}
+
+	// create 4 out of 2 tasks. no shutdown or failed tasks.  this would be the
+	// case if you did a call immediately after updating the service, before
+	// the orchestrator had updated the task desired states
+	for i := 0; i < 4; i++ {
+		createTask(over, running, running)
+	}
+
+	// create 2 running tasks for replicatedJob1
+	for i := 0; i < 2; i++ {
+		createTask(replicatedJob1, running, completed, withJobIteration)
+	}
+
+	// create 4 completed tasks for replicatedJob1
+	for i := 0; i < 4; i++ {
+		createTask(replicatedJob1, completed, completed, withJobIteration)
+	}
+
+	// create 10 completed tasks for replicatedJob2
+	for i := 0; i < 10; i++ {
+		createTask(replicatedJob2, completed, completed, withJobIteration)
+	}
+
+	replicatedJob2Spec.Task.ForceUpdate++
+
+	// now update replicatedJob2, so JobIteration gets incremented
+	updateResp, updateErr := ts.Client.UpdateService(
+		context.Background(),
+		&api.UpdateServiceRequest{
+			ServiceID:      replicatedJob2.ID,
+			ServiceVersion: &replicatedJob2.Meta.Version,
+			Spec:           replicatedJob2Spec,
+		},
+	)
+	assert.NoError(t, updateErr)
+	assert.NotNil(t, updateResp)
+	replicatedJob2 = updateResp.Service
+
+	// and create 1 tasks out of 2
+	createTask(replicatedJob2, running, completed, withJobIteration)
+	// and 3 completed already
+	for i := 0; i < 3; i++ {
+		createTask(replicatedJob2, completed, completed, withJobIteration)
+	}
+
+	// create 5 running tasks for globalJob
+	for i := 0; i < 5; i++ {
+		createTask(globalJob, running, completed, withJobIteration)
+	}
+	// create 3 completed tasks
+	for i := 0; i < 3; i++ {
+		createTask(globalJob, completed, completed, withJobIteration)
+	}
+
+	// now, create a service that has already been deleted, but has dangling
+	// tasks
+	goneSpec := createSpec("gone", "image", 3)
+	gone := &api.Service{
+		ID:   identity.NewID(),
+		Spec: *goneSpec,
+	}
+
+	for i := 0; i < 3; i++ {
+		createTask(gone, running, shutdown)
+		createTask(gone, shutdown, shutdown)
+	}
+
+	// now list service statuses
+	r, err = ts.Client.ListServiceStatuses(
+		context.Background(),
+		&api.ListServiceStatusesRequest{Services: []string{
+			justRight.ID, notEnough.ID, global.ID, global2.ID,
+			replicatedJob1.ID, replicatedJob2.ID, globalJob.ID, over.ID, gone.ID,
+		}},
+	)
+	assert.NoError(t, err, "error getting service statuses")
+	assert.NotNil(t, r, "service status response is nil")
+	assert.Len(t, r.Statuses, 9)
+
+	expected := map[string]*api.ListServiceStatusesResponse_ServiceStatus{
+		"justRight": {
+			ServiceID:    justRight.ID,
+			DesiredTasks: 3,
+			RunningTasks: 3,
+		},
+		"notEnough": {
+			ServiceID:    notEnough.ID,
+			DesiredTasks: 7,
+			RunningTasks: 4,
+		},
+		"global": {
+			ServiceID:    global.ID,
+			DesiredTasks: 2,
+			RunningTasks: 2,
+		},
+		"global2": {
+			ServiceID:    global2.ID,
+			DesiredTasks: 5,
+			RunningTasks: 4,
+		},
+		"over": {
+			ServiceID:    over.ID,
+			DesiredTasks: 2,
+			RunningTasks: 4,
+		},
+		"replicatedJob1": {
+			ServiceID:      replicatedJob1.ID,
+			DesiredTasks:   2,
+			RunningTasks:   2,
+			CompletedTasks: 4,
+		},
+		"replicatedJob2": {
+			ServiceID:      replicatedJob2.ID,
+			DesiredTasks:   2,
+			RunningTasks:   1,
+			CompletedTasks: 3,
+		},
+		"globalJob": {
+			ServiceID:      globalJob.ID,
+			DesiredTasks:   5,
+			RunningTasks:   5,
+			CompletedTasks: 3,
+		},
+		"gone": {
+			ServiceID:    gone.ID,
+			DesiredTasks: 0,
+			RunningTasks: 3,
+		},
+	}
+
+	// compare expected and actual values. make sure all are used by keeping
+	// track of which we visited. i borrowed this pattern from
+	// assert.ElementsMatch, which is in a newer version of that library
+	visited := make([]bool, len(expected))
+	for name, expect := range expected {
+		found := false
+		for i := 0; i < len(r.Statuses); i++ {
+			if visited[i] {
+				continue
+			}
+			if reflect.DeepEqual(expect, r.Statuses[i]) {
+				visited[i] = true
+				found = true
+				break
+			}
+		}
+		if !found {
+			t.Errorf("did not find status for %v in response", name)
+		}
+	}
+}
+
+// TestJobService tests that if a job-mode service is created, the necessary
+// fields are all initialized to the correct values. Then, it tests that if a
+// job-mode service is updated, the fields are updated to correct values.
+func TestJobService(t *testing.T) {
+	ts := newTestServer(t)
+	defer ts.Stop()
+
+	// first, create a replicated job mode service spec
+	spec := &api.ServiceSpec{
+		Annotations: api.Annotations{
+			Name: "replicatedjob",
+		},
+		Task: api.TaskSpec{
+			Runtime: &api.TaskSpec_Container{
+				Container: &api.ContainerSpec{
+					Image: "image",
+				},
+			},
+		},
+		Mode: &api.ServiceSpec_ReplicatedJob{
+			ReplicatedJob: &api.ReplicatedJob{
+				MaxConcurrent:    3,
+				TotalCompletions: 9,
+			},
+		},
+	}
+
+	before := gogotypes.TimestampNow()
+	// now, create the service
+	resp, err := ts.Client.CreateService(
+		context.Background(), &api.CreateServiceRequest{
+			Spec: spec,
+		},
+	)
+	after := gogotypes.TimestampNow()
+
+	// ensure there are no errors
+	require.NoError(t, err)
+	// and assert that the response is valid
+	require.NotNil(t, resp)
+	require.NotNil(t, resp.Service)
+	// ensure that the service has a JobStatus set
+	require.NotNil(t, resp.Service.JobStatus, "expected JobStatus to not be nil")
+	// and ensure that JobStatus.JobIteration is set to 0, which is the default
+	require.Equal(
+		t, resp.Service.JobStatus.JobIteration.Index, uint64(0),
+		"expected JobIteration for new replicated job to be 0",
+	)
+	require.NotNil(t, resp.Service.JobStatus.LastExecution)
+	assert.True(t, resp.Service.JobStatus.LastExecution.Compare(before) >= 0,
+		"expected %v to be after %v", resp.Service.JobStatus.LastExecution, before,
+	)
+	assert.True(t, resp.Service.JobStatus.LastExecution.Compare(after) <= 0,
+		"expected %v to be before %v", resp.Service.JobStatus.LastExecution, after,
+	)
+
+	// now, repeat all of the above, but with a global job
+	gspec := spec.Copy()
+	gspec.Annotations.Name = "globaljob"
+	gspec.Mode = &api.ServiceSpec_GlobalJob{
+		GlobalJob: &api.GlobalJob{},
+	}
+	before = gogotypes.TimestampNow()
+	gresp, gerr := ts.Client.CreateService(
+		context.Background(), &api.CreateServiceRequest{
+			Spec: gspec,
+		},
+	)
+	after = gogotypes.TimestampNow()
+
+	require.NoError(t, gerr)
+	require.NotNil(t, gresp)
+	require.NotNil(t, gresp.Service)
+	require.NotNil(t, gresp.Service.JobStatus)
+	require.Equal(
+		t, gresp.Service.JobStatus.JobIteration.Index, uint64(0),
+		"expected JobIteration for new global job to be 0",
+	)
+	require.NotNil(t, gresp.Service.JobStatus.LastExecution)
+	assert.True(t, gresp.Service.JobStatus.LastExecution.Compare(before) >= 0,
+		"expected %v to be after %v", gresp.Service.JobStatus.LastExecution, before,
+	)
+	assert.True(t, gresp.Service.JobStatus.LastExecution.Compare(after) <= 0,
+		"expected %v to be before %v", gresp.Service.JobStatus.LastExecution, after,
+	)
+
+	// now test that updating the service increments the JobIteration
+	spec.Task.ForceUpdate = spec.Task.ForceUpdate + 1
+	before = gogotypes.TimestampNow()
+	uresp, uerr := ts.Client.UpdateService(
+		context.Background(), &api.UpdateServiceRequest{
+			ServiceID:      resp.Service.ID,
+			ServiceVersion: &(resp.Service.Meta.Version),
+			Spec:           spec,
+		},
+	)
+	after = gogotypes.TimestampNow()
+
+	require.NoError(t, uerr)
+	require.NotNil(t, uresp)
+	require.NotNil(t, uresp.Service)
+	require.NotNil(t, uresp.Service.JobStatus)
+	// updating the service should bump the JobStatus.JobIteration.Index by 1
+	require.Equal(
+		t, uresp.Service.JobStatus.JobIteration.Index, uint64(1),
+		"expected JobIteration for updated replicated job to be 1",
+	)
+	require.NotNil(t, uresp.Service.JobStatus.LastExecution)
+	assert.True(t, uresp.Service.JobStatus.LastExecution.Compare(before) >= 0,
+		"expected %v to be after %v", uresp.Service.JobStatus.LastExecution, before,
+	)
+	assert.True(t, uresp.Service.JobStatus.LastExecution.Compare(after) <= 0,
+		"expected %v to be before %v", uresp.Service.JobStatus.LastExecution, after,
+	)
+
+	// rinse and repeat
+	gspec.Task.ForceUpdate = spec.Task.ForceUpdate + 1
+	before = gogotypes.TimestampNow()
+	guresp, guerr := ts.Client.UpdateService(
+		context.Background(), &api.UpdateServiceRequest{
+			ServiceID:      gresp.Service.ID,
+			ServiceVersion: &(gresp.Service.Meta.Version),
+			Spec:           gspec,
+		},
+	)
+	after = gogotypes.TimestampNow()
+
+	require.NoError(t, guerr)
+	require.NotNil(t, guresp)
+	require.NotNil(t, guresp.Service)
+	require.NotNil(t, guresp.Service.JobStatus)
+	require.Equal(
+		t, guresp.Service.JobStatus.JobIteration.Index, uint64(1),
+		"expected JobIteration for updated replicated job to be 1",
+	)
+	require.NotNil(t, guresp.Service.JobStatus.LastExecution)
+	assert.True(t, guresp.Service.JobStatus.LastExecution.Compare(before) >= 0,
+		"expected %v to be after %v", guresp.Service.JobStatus.LastExecution, before,
+	)
+	assert.True(t, guresp.Service.JobStatus.LastExecution.Compare(after) <= 0,
+		"expected %v to be before %v", guresp.Service.JobStatus.LastExecution, after,
+	)
+}
+
+// TestServiceValidateJob tests that calling the service API correctly
+// validates a job-mode service. Some fields, like UpdateConfig, are not valid
+// for job-mode services, and should return an error if they're set
+func TestServiceValidateJob(t *testing.T) {
+	bad := &api.ServiceSpec{
+		Mode:   &api.ServiceSpec_ReplicatedJob{ReplicatedJob: &api.ReplicatedJob{}},
+		Update: &api.UpdateConfig{},
+	}
+
+	err := validateJob(bad)
+	assert.Error(t, err)
+	assert.Equal(t, codes.InvalidArgument, testutils.ErrorCode(err))
+
+	good := []*api.ServiceSpec{
+		{
+			Mode: &api.ServiceSpec_ReplicatedJob{
+				ReplicatedJob: &api.ReplicatedJob{
+					MaxConcurrent: 3, TotalCompletions: 9,
+				},
+			},
+		},
+		{Mode: &api.ServiceSpec_GlobalJob{}},
+	}
+
+	for _, g := range good {
+		err := validateJob(g)
+		assert.NoError(t, err)
+	}
+}
