@@ -1,11 +1,10 @@
 package transport
 
 import (
+	"context"
 	"math"
 	"testing"
 
-	"golang.org/x/net/context"
-
 	"github.com/coreos/etcd/raft/raftpb"
 	"github.com/stretchr/testify/assert"
 )
